//modified by Jing Yang after 7/6/2001
#include "CSunburstClusterTree.H"
#include "stdio.h"
#include "XmdvTool.H"
#include <string.h>

CSunburstClusterTree::CSunburstClusterTree()
{
//	dim_cluster_file = NULL;
    CMAP = colormap(true);
	current_list = NULL;
}


CSunburstClusterTree::~CSunburstClusterTree()
{
    delete root;
}

//construct the cluster tree for sunburst display according to the arrays;
//parameters: int num_nodes-number of nodes.
//parameters: array nodes-nodes stored in an array according to their ids. nodes[0] must be the root node.
//parameters: array parent_ids-corresponding nodes' parent' ids. nodes[0](the root node)'s parent id is 
//-1. Other parent ids should be larger than -1.
bool CSunburstClusterTree::ConstructSunburstClusterTree(int num_nodes, ARR<CSunburstCluster *> nodes, ARR<int> parent_ids)
{
	//if can't find the root node, return false
	if ( parent_ids[0] != -1 || nodes[0] == NULL)	
		return false;
	
	//link the clusters into a tree
	root = nodes[0];
	for ( int i = 1; i < num_nodes; i++ )
	{
		if ( parent_ids[i] < 0 || parent_ids[i] >= num_nodes )
			return false;
		if ( nodes[parent_ids[i]] == NULL )
			return false;
		nodes[parent_ids[i]]->AddChild(nodes[i]);
	}

	ResetLevels();
	ResetAngles();
	ResetColors();
	return true;
}

//set level for every node in the tree, find the max_level of the tree
void CSunburstClusterTree::ResetLevels()
{
	max_level = 1;
	root->ResetLevels(1, max_level);
}

void CSunburstClusterTree::ResetEntries()
{
    root->ResetEntries();
}


//reset color of all nodes of the tree. color ~ mid angle of the node
void CSunburstClusterTree::ResetColors()
{
	root->ResetColors(CMAP);
}


//set start_angles and sweep_angles for all the nodes
void CSunburstClusterTree::ResetAngles()
{
	unit_angle = 360. / root->entries;
	mini_angle = unit_angle / 25;
	root->ResetAngles(0, unit_angle);
}


CSunburstCluster* CSunburstClusterTree::FindNodeAccordingLevelAndAngle(int level, double angle)
{
	if ( level < root->level )
		return NULL;

	while ( angle < 0 )
		angle += 360;
	while ( angle > 360 )
		angle -= 360;

	return root->FindNodeAccordingLevelAndAngle(level, angle);
}


CSunburstCluster* CSunburstClusterTree::FindNodeAccordingLevelAndAngle(int level, double angle, bool &start, bool &end, bool &in)
{
	if ( level < root->level )
		return NULL;

	while ( angle < 0 )
		angle += 360;
	while ( angle > 360 )
		angle -= 360;

	start = false;
	end = false;
	in = false;

	return root->FindNodeAccordingLevelAndAngle(level, angle, mini_angle/2, start, end, in);
}


void CSunburstClusterTree::ResetCurrentList()
{
	if ( current_list != NULL )
		delete current_list;

	current_list = new CLinkList();
	root->ResetCurrentList(current_list);
}


void CSunburstClusterTree::Rotate(double degree)
{
	root->Rotate(degree);
}