#ifndef BUFFER_H
#define BUFFER_H

#include "db_TABLE.h"


#ifdef _WIN32
	#include <windows.h>
#endif

extern int	NO_NODES, NO_P_ENTRIES, NO_L_ENTRIES;
extern double	old_x, old_y, old_l;

//jing
#ifndef _WIN32
	extern pthread_mutex_t mutex;
#else
	extern HANDLE mutex;
#endif


#ifndef		NULL
	#define		NULL		0
#endif


/*  database entry   */

typedef struct DB_ENTRY_PTR {

	/* cluster's extents and level	*/
	double	X,					// Left  extent
			Y,					// Right extent
			L;					// Level extent
  
	int		N;					// datapoints in cluster
	double        D[TABLE_ND];	// dim. aggregate values

} DB_ENTRY;


/*  buffer entry     */

typedef struct ENTRY_PTR {

	int					P;		// probability bucket value
	int					L;		// level       bucket value
	DB_ENTRY			*data;	// database entries
	struct ENTRY_PTR	*nextL;	// next     level       entry
	struct ENTRY_PTR	*prevL;	// previous level       entry
	struct ENTRY_PTR	*nextP;	// next     probability entry
	struct ENTRY_PTR	*prevP;	// previous probability entry


} ENTRY;


/*	bucket tables		*/

typedef struct T_ENTRY_PTR {

	ENTRY		*first;			// first entry in the bucket
	ENTRY		*last;			// last  entry in the bucket

} T_ENTRY;


/*   Function prototypes   */

extern void destroyBackendConnectivity();
extern void      error(char *);
extern void      init_memory(char *name);
extern void      init_btable(T_ENTRY *t);
extern ENTRY    *new_entry();
extern DB_ENTRY *new_db_entry();
extern void      P_hash(ENTRY *e);
extern void      L_hash(ENTRY *e);
extern void      save_active_set();
extern void      register_change();
extern int       prepare(double x, double y, double l, int pi);
extern void      init();
extern ENTRY    *get_one();

#endif
